@@ -17,8 +17,6 @@ use aliased 'Bio::MUST::Core::IdList';
 use aliased 'Bio::MUST::Core::SeqId';
 use aliased 'Bio::MUST::Core::Taxonomy';
 
-# TODO: check values of arg --item-type (debug Gepopt::Euclid and --item?)
-# TODO: allow for multi-column input with --item-col argument
 
 # build taxonomy object
 my $tax = Taxonomy->new_from_cache( tax_dir => $ARGV_taxdir );
@@ -36,8 +34,8 @@ if ($ARGV_org_mapper) {
     $suffix = '.org-idm';
 }
 if ($ARGV_legacy_nom) {
-    $ARGV_noitem    = 1;
-    $ARGV_nomustid  = 0;
+    $ARGV_noitem    = 0;                    # give precedence to items
+    $ARGV_nomustid  = 1;                    # to ensure back-compatibility
     $ARGV_notaxid   = 1;
     $ARGV_nolineage = 0;
     $suffix = '.nom';
@@ -57,35 +55,41 @@ if (@ARGV_levels) {
 
 ### --item-type: $ARGV_item_type
 
-# method and anon sub dispatch tables...
+# method, args and anon sub dispatch tables...
 # ... for reading infiles
 my $method = $ARGV_from_must ? 'load_lis' : 'load';
+my $args = {
+    column    => $ARGV_column - 1,      # 0-based numbering in lib
+    separator => $ARGV_separator,
+};
 
 # .. and for fetching taxon_ids from items
 my %fetch_from = (
-    mustid => sub { map { $tax->get_taxid_from_seq_id($_)   } @_ },
-    baseid => sub { map { $tax->get_taxid_from_seq_id($_)   }
-                    map { $_ . '@1'                         } @_ },
+    mustid  => sub { map { $tax->get_taxid_from_seq_id(   $_) } @_ },
+    baseid  => sub { map { $tax->get_taxid_from_seq_id(   $_) }
+                     map { $_ . '@1'                          } @_ },
             # we build a mustid from the baseid
-    strain => sub { map { $tax->get_taxid_from_seq_id($_)   }
-                    map { SeqId->new_with(
+    strain  => sub { map { $tax->get_taxid_from_seq_id(   $_) }
+                     map { SeqId->new_with(
                             org         => $_,
                             accession   => 1,
                             keep_strain => 1,
-                        )                                   } @_ },
+                         )                                    } @_ },
             # we build a mustid from the strain name
-    gi     => sub { map { $tax->get_taxid_from_seq_id($_)   }
-                    map { $_ =~ $PKEYONLY ? 'gi|' . $_ : $_ } @_ },
-            # we let ..._from_seq_id doing the GI number parsing
-    name   => sub { map { $tax->get_taxid_from_name(  $_)   } @_ },
+    name    => sub { map { $tax->get_taxid_from_name(     $_) } @_ },
             # we directly use ..._from_name to use the whole item
-    taxid  => sub { @_ },
+    lineage => sub { map { $tax->get_taxid_from_taxonomy( $_) } @_ },
+            # we use the "ambiguous-taxa resistant" method based on lineages
+    gi      => sub { map { $tax->get_taxid_from_seq_id(   $_) }
+                     map { $_ =~ $PKEYONLY ? 'gi|' . $_ : $_  } @_ },
+            # we let ..._from_seq_id doing the GI number parsing
+    taxid   => sub { @_ },
 );
 
 for my $infile (@ARGV_infiles) {
 
     ### Processing: $infile
-    my $list = IdList->$method($infile);
+    my $list = IdList->$method($infile, $args);
 
     # fetch and clean up list items
     # Note: using apply instead of map for satisfying Perl::Critic
@@ -169,7 +173,7 @@ fetch-tax.pl - Fetch information from the NCBI Taxonomy database
 
 =head1 VERSION
 
-version 0.200510
+version 0.201060
 
 =head1 USAGE
 
@@ -203,17 +207,33 @@ Path to local mirror of the NCBI Taxonomy database.
 Consider the input file as generated by ed/treeplot [default: no]. Currently,
 this switches to the legacy .lis format (instead of the modern .idl format).
 
+=item --col[umn]=<n>
+
+Column number providing the string to be used as the item [default: 1]. Columns
+are numbered as would do the shell, i.e., they start at 1.
+
+=for Euclid: n.type:    +int
+    n.default: 1
+
+=item --sep[arator]=<str>
+
+Separator used to split columns [default: '\t'].
+
+=for Euclid: str.type:    string
+    str.default: "\t"
+
 =item --item-type=<str>
 
 Type of the items listed in the infile [default: mustid]. The following
 types are available:
 
-    - mustid (standard MUST ids, including the '@')
-    - baseid (base MUST ids, truncated before the '@')
-    - strain (catalog strains to coerce to NCBI names)
-    - name   (NCBI names)
-    - taxid  (NCBI taxon ids or GCA/GCF accessions)
-    - gi     (NCBI GIs, complete accessions are allowed)
+    - mustid  (standard MUST ids, including the '@')
+    - baseid  (base MUST ids, truncated before the '@')
+    - strain  (catalog strains to coerce to NCBI names)
+    - name    (NCBI names)
+    - lineage (NCBI (or SILVA) lineages separated by ';')
+    - taxid   (NCBI taxon ids or GCA/GCF accessions)
+    - gi      (NCBI GIs, complete accessions are allowed)
 
 C<mustid> and C<baseid> items will both be analyzed by the
 L<Bio::MUST::Core::Taxonomy> heuristics, whereas C<name> items will be
@@ -233,8 +253,8 @@ Using C<gi> items requires having installed the GI-to-taxid mapper during
 setup of the local mirror of the NCBI Taxonomy database (see
 L<setup-taxdir.pl> for details).
 
-=for Euclid: str.type:       /mustid|baseid|strain|name|taxid|gi/
-    str.type.error: <str> must be one of mustid, baseid, strain, name, taxid or gi (not str)
+=for Euclid: str.type:       /mustid|baseid|strain|name|lineage|taxid|gi/
+    str.type.error: <str> must be one of mustid, baseid, strain, name, lineage, taxid or gi (not str)
     str.default:    'mustid'
 
 =item --keep-strain
